@@ -1,17 +1,17 @@
-# Copyright 2010-2011 Red Hat, Inc.
+# Copyright 2010-2019 Red Hat, Inc.
 #
 # This copyrighted material is made available to anyone wishing to use,
 # modify, copy, or redistribute it subject to the terms and conditions
 # of the GNU General Public License v.2.
 
-from distutils.core import setup, Extension
+from setuptools import setup, Extension
 
 sanlocklib = ['sanlock']
 sanlock = Extension(name='sanlock',
                     sources=['sanlock.c'],
                     include_dirs=['../src'],
                     library_dirs=['../src'],
-                    extra_link_args=['-fPIE', '-Wl,-z,relro,-z,now'],
+                    extra_compile_args=["-std=c99"],
                     libraries=sanlocklib)
 
 version = None
