@@ -23,6 +23,7 @@
 #include <sys/time.h>
 
 #include "sanlock_internal.h"
+#include "sanlock_admin.h"
 #include "diskio.h"
 #include "ondisk.h"
 #include "log.h"
@@ -31,6 +32,51 @@
 #include "paxos_lease.h"
 #include "delta_lease.h"
 #include "timeouts.h"
+#include "rindex.h"
+
+/*
+ * the caller sets sd.offset to the location from the start of disk (in bytes) where
+ * a data struct should be read and checked for sector/align sizes.
+ */
+static int direct_read_leader_sizes(struct task *task, struct sync_disk *sd,
+				    int *sector_size, int *align_size)
+{
+	struct leader_record *lr_end;
+	struct leader_record lr_in;
+	char *data;
+	int datalen;
+	int rv;
+
+	datalen = 4096;
+	data = malloc(datalen);
+
+	if (!data)
+		return -ENOMEM;
+
+	memset(data, 0, datalen);
+
+	rv = read_sectors(sd, 4096, 0, 1, data, datalen, task, DEFAULT_IO_TIMEOUT, "read_sector_size");
+	if (rv < 0) {
+		free(data);
+		return rv;
+	}
+
+	lr_end = (struct leader_record *)data;
+
+	leader_record_in(lr_end, &lr_in);
+
+	free(data);
+
+	if ((lr_in.magic == DELTA_DISK_MAGIC) || (lr_in.magic == PAXOS_DISK_MAGIC)) {
+		*sector_size = lr_in.sector_size;
+		*align_size = leader_align_size_from_flag(lr_in.flags);
+		if (!*align_size)
+			*align_size = sector_size_to_align_size_old(*sector_size);
+		return 0;
+	}
+
+	return -1;
+}
 
 /*
  * cli: sanlock direct init
@@ -74,19 +120,26 @@
  */
 
 static int do_paxos_action(int action, struct task *task, int io_timeout, struct sanlk_resource *res,
-			   int max_hosts, int num_hosts, int write_clear,
+			   int num_hosts, int write_clear,
 			   uint64_t local_host_id, uint64_t local_host_generation,
 			   struct leader_record *leader_in, struct leader_record *leader_ret)
 {
 	struct token *token;
 	struct leader_record leader;
 	struct paxos_dblock dblock;
+	int sector_size = 0;
+	int align_size = 0;
+	int max_hosts = 0;
 	int disks_len, token_len;
 	int j, rv = 0;
 
 	if (!io_timeout)
 		io_timeout = DEFAULT_IO_TIMEOUT;
 
+	rv = sizes_from_flags(res->flags, &sector_size, &align_size, &max_hosts, "RES");
+	if (rv)
+		return -1;
+
 	disks_len = res->num_disks * sizeof(struct sync_disk);
 	token_len = sizeof(struct token) + disks_len;
 
@@ -99,6 +152,7 @@ static int do_paxos_action(int action, struct task *task, int io_timeout, struct
 	token->r.num_disks = res->num_disks;
 	memcpy(token->r.lockspace_name, res->lockspace_name, SANLK_NAME_LEN);
 	memcpy(token->r.name, res->name, SANLK_NAME_LEN);
+	token->r.flags = res->flags;
 
 	/* WARNING sync_disk == sanlk_disk */
 
@@ -109,36 +163,86 @@ static int do_paxos_action(int action, struct task *task, int io_timeout, struct
 		token->disks[j].fd = -1;
 	}
 
+
 	rv = open_disks(token->disks, token->r.num_disks);
 	if (rv < 0) {
 		free(token);
 		return rv;
 	}
 
+	if (!sector_size && com.sector_size)
+		sector_size = com.sector_size;
+
+	if (!align_size && com.align_size)
+		align_size = com.align_size;
+
 	switch (action) {
 	case ACT_DIRECT_INIT:
-		rv = paxos_lease_init(task, token, num_hosts, max_hosts, write_clear);
+		/* paxos_lease_init looks at token->r.flags for sector/align flags */
+
+		rv = paxos_lease_init(task, token, num_hosts, write_clear);
 		break;
 
 	case ACT_ACQUIRE:
+		if (!sector_size || !align_size) {
+			rv = direct_read_leader_sizes(task, &token->disks[0], &sector_size, &align_size);
+			if (rv < 0)
+				break;
+		}
+		token->sector_size = sector_size;
+		token->align_size = align_size;
+
 		token->host_id = local_host_id;
 		token->host_generation = local_host_generation;
 
-		rv = paxos_lease_acquire(task, token, 0, leader_ret, &dblock, 0, num_hosts);
+		rv = paxos_lease_acquire(task, token, 0, leader_ret, &dblock, 0, 0);
 		break;
 
 	case ACT_RELEASE:
+		if (!sector_size)
+			sector_size = 4096;
+		if (!align_size)
+			align_size = sector_size_to_align_size_old(sector_size);
+
+		token->sector_size = sector_size;
+		token->align_size = align_size;
+
 		rv = paxos_lease_leader_read(task, token, &leader, "direct_release");
 		if (rv < 0)
 			break;
+
+		sector_size = leader.sector_size;
+		align_size = leader_align_size_from_flag(leader.flags);
+		if (!align_size)
+			align_size = sector_size_to_align_size_old(sector_size);
+
+		token->sector_size = sector_size;
+		token->align_size = align_size;
+
 		rv = paxos_lease_release(task, token, NULL, &leader, leader_ret);
 		break;
 
 	case ACT_READ_LEADER:
+		if (!sector_size)
+			sector_size = 4096;
+		if (!align_size)
+			align_size = sector_size_to_align_size_old(sector_size);
+
+		token->sector_size = sector_size;
+		token->align_size = align_size;
+
 		rv = paxos_lease_leader_read(task, token, &leader, "direct_read_leader");
 		break;
 
 	case ACT_WRITE_LEADER:
+		sector_size = leader_in->sector_size;
+		align_size = leader_align_size_from_flag(leader_in->flags);
+		if (!align_size)
+			align_size = sector_size_to_align_size_old(sector_size);
+
+		token->sector_size = sector_size;
+		token->align_size = align_size;
+
 		rv = paxos_lease_leader_clobber(task, token, leader_in, "direct_clobber");
 		break;
 	}
@@ -168,7 +272,7 @@ int direct_acquire(struct task *task, int io_timeout,
 		   struct leader_record *leader_ret)
 {
 	return do_paxos_action(ACT_ACQUIRE, task, io_timeout, res,
-			       -1, num_hosts, 0,
+			       num_hosts, 0,
 			       local_host_id, local_host_generation,
 			       NULL, leader_ret);
 }
@@ -178,7 +282,7 @@ int direct_release(struct task *task, int io_timeout,
 		   struct leader_record *leader_ret)
 {
 	return do_paxos_action(ACT_RELEASE, task, io_timeout, res,
-			       -1, -1, 0,
+			       0, 0,
 			       0, 0,
 			       NULL, leader_ret);
 }
@@ -187,7 +291,6 @@ static int do_delta_action(int action,
 			   struct task *task,
 			   int io_timeout,
 			   struct sanlk_lockspace *ls,
-			   int max_hosts,
 			   char *our_host_name,
 			   struct leader_record *leader_in,
 			   struct leader_record *leader_ret)
@@ -196,14 +299,24 @@ static int do_delta_action(int action,
 	struct sync_disk sd;
 	struct space space;
 	char bitmap[HOSTID_BITMAP_SIZE];
-	int read_result, rv;
+	int sector_size = 0;
+	int align_size = 0;
+	int max_hosts = 0;
+	int read_result;
 	int rd_ms, wr_ms;
+	int rv;
 
 	memset(bitmap, 0, sizeof(bitmap));
 
 	if (!io_timeout)
 		io_timeout = DEFAULT_IO_TIMEOUT;
 
+	rv = sizes_from_flags(ls->flags, &sector_size, &align_size, &max_hosts, "LSF");
+	if (rv)
+		return -1;
+
+	memset(&leader, 0, sizeof(leader));
+
 	/* for log_space in delta functions */
 	memset(&space, 0, sizeof(space));
 	space.io_timeout = io_timeout;
@@ -222,12 +335,29 @@ static int do_delta_action(int action,
 	if (rv < 0)
 		return -ENODEV;
 
+	if (!sector_size && com.sector_size)
+		sector_size = com.sector_size;
+
+	if (!align_size && com.align_size)
+		align_size = com.align_size;
+
 	switch (action) {
 	case ACT_DIRECT_INIT:
-		rv = delta_lease_init(task, io_timeout, &sd, ls->name, max_hosts);
+		/* delta_lease_init looks at ls->flags for sector/align sizes */
+
+		rv = delta_lease_init(task, ls, io_timeout, &sd);
 		break;
 
 	case ACT_ACQUIRE_ID:
+		if (!sector_size || !align_size) {
+			rv = direct_read_leader_sizes(task, &sd, &sector_size, &align_size);
+			if (rv < 0)
+				break;
+		}
+
+		space.sector_size = sector_size;
+		space.align_size = align_size;
+
 		rv = delta_lease_acquire(task, &space, &sd,
 					 ls->name,
 					 our_host_name,
@@ -235,13 +365,23 @@ static int do_delta_action(int action,
 					 &leader);
 		break;
 	case ACT_RENEW_ID:
-		rv = delta_lease_leader_read(task, io_timeout, &sd,
+		if (!sector_size || !align_size) {
+			rv = direct_read_leader_sizes(task, &sd, &sector_size, &align_size);
+			if (rv < 0)
+				break;
+		}
+
+		space.sector_size = sector_size;
+		space.align_size = align_size;
+
+		rv = delta_lease_leader_read(task, sector_size, io_timeout, &sd,
 					     ls->name,
 					     ls->host_id,
 					     &leader,
 					     "direct_renew");
 		if (rv < 0)
 			return rv;
+
 		rv = delta_lease_renew(task, &space, &sd,
 				       ls->name,
 				       bitmap,
@@ -254,20 +394,36 @@ static int do_delta_action(int action,
 				       &rd_ms, &wr_ms);
 		break;
 	case ACT_RELEASE_ID:
-		rv = delta_lease_leader_read(task, io_timeout, &sd,
+		if (!sector_size || !align_size) {
+			rv = direct_read_leader_sizes(task, &sd, &sector_size, &align_size);
+			if (rv < 0)
+				break;
+		}
+
+		space.sector_size = sector_size;
+		space.align_size = align_size;
+
+		rv = delta_lease_leader_read(task, sector_size, io_timeout, &sd,
 					     ls->name,
 					     ls->host_id,
 					     &leader,
 					     "direct_release");
 		if (rv < 0)
 			return rv;
+
 		rv = delta_lease_release(task, &space, &sd,
 					 ls->name,
 					 &leader,
 					 &leader);
 		break;
 	case ACT_READ_LEADER:
-		rv = delta_lease_leader_read(task, io_timeout, &sd,
+		if (!sector_size || !align_size) {
+			rv = direct_read_leader_sizes(task, &sd, &sector_size, &align_size);
+			if (rv < 0)
+				break;
+		}
+
+		rv = delta_lease_leader_read(task, sector_size, io_timeout, &sd,
 					     ls->name,
 					     ls->host_id,
 					     &leader,
@@ -301,17 +457,17 @@ static int do_delta_action(int action,
 int direct_acquire_id(struct task *task, int io_timeout, struct sanlk_lockspace *ls,
 		      char *our_host_name)
 {
-	return do_delta_action(ACT_ACQUIRE_ID, task, io_timeout, ls, -1, our_host_name, NULL, NULL);
+	return do_delta_action(ACT_ACQUIRE_ID, task, io_timeout, ls, our_host_name, NULL, NULL);
 }
 
 int direct_release_id(struct task *task, int io_timeout, struct sanlk_lockspace *ls)
 {
-	return do_delta_action(ACT_RELEASE_ID, task, io_timeout, ls, -1, NULL, NULL, NULL);
+	return do_delta_action(ACT_RELEASE_ID, task, io_timeout, ls, NULL, NULL, NULL);
 }
 
 int direct_renew_id(struct task *task, int io_timeout, struct sanlk_lockspace *ls)
 {
-	return do_delta_action(ACT_RENEW_ID, task, io_timeout, ls, -1, NULL, NULL, NULL);
+	return do_delta_action(ACT_RENEW_ID, task, io_timeout, ls, NULL, NULL, NULL);
 }
 
 int direct_align(struct sync_disk *disk)
@@ -326,17 +482,16 @@ int direct_align(struct sync_disk *disk)
 
 /* io_timeout is written to leader record and used for the write call itself */
 int direct_write_lockspace(struct task *task, struct sanlk_lockspace *ls,
-			   int max_hosts, uint32_t io_timeout)
+			   uint32_t io_timeout)
 {
 	if (!ls)
 		return -1;
 
-	return do_delta_action(ACT_DIRECT_INIT, task, io_timeout, ls,
-			       max_hosts, NULL, NULL, NULL);
+	return do_delta_action(ACT_DIRECT_INIT, task, io_timeout, ls, NULL, NULL, NULL);
 }
 
 int direct_write_resource(struct task *task, struct sanlk_resource *res,
-			  int max_hosts, int num_hosts, int write_clear)
+			  int num_hosts, int write_clear)
 {
 	if (!res)
 		return -1;
@@ -348,7 +503,7 @@ int direct_write_resource(struct task *task, struct sanlk_resource *res,
 		return -ENODEV;
 
 	return do_paxos_action(ACT_DIRECT_INIT, task, 0, res,
-			       max_hosts, num_hosts, write_clear,
+			       num_hosts, write_clear,
 			       0, 0,
 			       NULL, NULL);
 }
@@ -362,11 +517,11 @@ int direct_read_leader(struct task *task,
 	int rv = -1;
 
 	if (ls && ls->host_id_disk.path[0])
-		rv = do_delta_action(ACT_READ_LEADER, task, io_timeout, ls, -1, NULL, NULL, leader_ret);
+		rv = do_delta_action(ACT_READ_LEADER, task, io_timeout, ls, NULL, NULL, leader_ret);
 
 	else if (res)
 		rv = do_paxos_action(ACT_READ_LEADER, task, io_timeout, res,
-				     -1, -1, 0,
+				     0, 0,
 				     0, 0,
 				     NULL, leader_ret);
 	return rv;
@@ -381,11 +536,11 @@ int direct_write_leader(struct task *task,
 	int rv = -1;
 
 	if (ls && ls->host_id_disk.path[0]) {
-		rv = do_delta_action(ACT_WRITE_LEADER, task, io_timeout, ls, -1, NULL, leader, NULL);
+		rv = do_delta_action(ACT_WRITE_LEADER, task, io_timeout, ls, NULL, leader, NULL);
 
 	} else if (res) {
 		rv = do_paxos_action(ACT_WRITE_LEADER, task, io_timeout, res,
-				     -1, -1, 0,
+				     0, 0,
 				     0, 0,
 				     leader, NULL);
 	}
@@ -398,7 +553,14 @@ int test_id_bit(int host_id, char *bitmap);
 int direct_dump(struct task *task, char *dump_path, int force_mode)
 {
 	char *data, *bitmap;
-	char *colon, *off_str;
+	char *colon1 = NULL, *colon2 = NULL, *off_str = NULL, *size_str = NULL, *m;
+	uint32_t magic;
+	struct rindex_header *rh_end;
+	struct rindex_header *rh;
+	struct rindex_header rh_in;
+	struct rindex_entry *re_end;
+	struct rindex_entry *re;
+	struct rindex_entry re_in;
 	struct leader_record *lr_end;
 	struct leader_record *lr;
 	struct leader_record lr_in;
@@ -409,45 +571,115 @@ int direct_dump(struct task *task, char *dump_path, int force_mode)
 	char sname[NAME_ID_SIZE+1];
 	char rname[NAME_ID_SIZE+1];
 	uint64_t sector_nr;
+	uint64_t start_offset = 0;
 	uint64_t dump_size = 0;
-	uint64_t end_sector_nr;
-	int sector_count, datalen, align_size;
-	int i, rv, b;
+	uint64_t end_sector_nr = 0;
+	int sector_size = 0;
+	int align_size = 0;
+	int sector_count, datalen, max_hosts;
+	int i, j, rv, b;
 
 	memset(&sd, 0, sizeof(struct sync_disk));
 
-	/* /path[:<offset>[:<size>]] */
-	colon = strstr(dump_path, ":");
-	if (colon) {
-		off_str = colon + 1;
-		*colon = '\0';
-		sd.offset = atoll(off_str);
+	/*
+	 * /path[:<offset>[:<size>]]
+	 *
+	 * If path contains a colon, the user would escape it with \\, e.g.
+	 * device named /dev/foo:32 using offset 0 and lenth 1M would be
+	 * /dev/foo\\:32:0:1M
+	 */
 
-		colon = strstr(off_str, ":");
-		if (colon)
-			dump_size = atoll(colon + 1);
+	for (i = 0; i < strlen(dump_path); i++) {
+		if (dump_path[i] == '\\') {
+			i++;
+			continue;
+		}
+
+		if (dump_path[i] == ':') {
+			if (!colon1)
+				colon1 = &dump_path[i];
+			else if (!colon2)
+				colon2 = &dump_path[i];
+		}
 	}
 
-	strncpy(sd.path, dump_path, SANLK_PATH_LEN);
+	if (colon1) {
+		*colon1 = '\0';
+		off_str = colon1 + 1;
+
+		if (colon2) {
+			*colon2 = '\0';
+			size_str = colon2 + 1;
+		}
+
+		if ((m = strchr(off_str, 'M'))) {
+			*m = '\0';
+			start_offset = atoll(off_str) * 1024 * 1024;
+		} else {
+                        start_offset = atoll(off_str);
+		}
+
+		if (size_str) {
+			if ((m = strchr(size_str, 'M'))) {
+				*m = '\0';
+				dump_size = atoll(size_str) * 1024 * 1024;
+			} else {
+				dump_size = atoll(size_str);
+			}
+		}
+	}
+
+	if (start_offset % 1048576)
+		printf("WARNING: dump offset should be a multiple of 1048576 bytes.\n");
+
+	sanlock_path_import(sd.path, dump_path, sizeof(sd.path));
+
 	sd.fd = -1;
 
 	rv = open_disk(&sd);
-	if (rv < 0)
+	if (rv < 0) {
+		printf("Device %s not found.\n", sd.path);
 		return -ENODEV;
+	}
 
-	rv = direct_align(&sd);
-	if (rv < 0)
-		goto out_close;
+	if (com.sector_size)
+		sector_size = com.sector_size;
+        if (com.align_size)
+                align_size = com.align_size;
 
-	align_size = rv;
+	if (!sector_size || !align_size) {
+		sd.offset = start_offset;
+		for (i = 0; i < 1024; i++) {
+			rv = direct_read_leader_sizes(task, &sd, &sector_size, &align_size);
+			if (sector_size && align_size)
+				break;
+
+			/*
+			 * search for a data structure that contains sector_size/align_size
+			 * every 1MB, up to 1GB or dump_size.
+			 */
+			sd.offset += 1048576;
+
+			if (dump_size && (sd.offset >= (start_offset + dump_size)))
+				break;
+		}
+		if (!sector_size || !align_size) {
+			printf("Cannot find sector_size and align_size, set with -A and -Z.\n");
+			goto out_close;
+		}
+	}
+
+	max_hosts = size_to_max_hosts(sector_size, align_size);
+
+	sector_count = align_size / sector_size;
 	datalen = align_size;
-	sector_count = align_size / sd.sector_size;
 
 	data = malloc(datalen);
 	if (!data) {
 		rv = -ENOMEM;
 		goto out_close;
 	}
+	memset(data, 0, datalen);
 
 	printf("%8s %36s %48s %10s %4s %4s %s",
 	       "offset",
@@ -463,25 +695,28 @@ int direct_dump(struct task *task, char *dump_path, int force_mode)
 
 	printf("\n");
 
+	sd.offset = start_offset;
 	sector_nr = 0;
-	end_sector_nr = dump_size / sd.sector_size;
+	if (dump_size)
+		end_sector_nr = dump_size / sector_size;
 
 	while (end_sector_nr == 0 || sector_nr < end_sector_nr) {
 		memset(sname, 0, sizeof(rname));
 		memset(rname, 0, sizeof(rname));
-		memset(data, 0, sd.sector_size);
+		memset(data, 0, sector_size);
 
-		rv = read_sectors(&sd, sector_nr, sector_count, data, datalen,
+		rv = read_sectors(&sd, sector_size, sector_nr, sector_count, data, datalen,
 				  task, DEFAULT_IO_TIMEOUT, "dump");
 
-		lr_end = (struct leader_record *)data;
+		magic_in(data, &magic);
 
-		leader_record_in(lr_end, &lr_in);
-		lr = &lr_in;
+		if (magic == DELTA_DISK_MAGIC) {
+			lr_end = (struct leader_record *)data;
+			leader_record_in(lr_end, &lr_in);
+			lr = &lr_in;
 
-		if (lr->magic == DELTA_DISK_MAGIC) {
 			for (i = 0; i < sector_count; i++) {
-				lr_end = (struct leader_record *)(data + (i * sd.sector_size));
+				lr_end = (struct leader_record *)(data + (i * sector_size));
 
 				if (!lr_end->magic)
 					continue;
@@ -497,7 +732,7 @@ int direct_dump(struct task *task, char *dump_path, int force_mode)
 				strncpy(rname, lr->resource_name, NAME_ID_SIZE);
 
 				printf("%08llu %36s %48s %010llu %04llu %04llu",
-					(unsigned long long)((sector_nr + i) * sd.sector_size),
+					(unsigned long long)(start_offset + ((sector_nr + i) * sector_size)),
 					sname, rname,
 					(unsigned long long)lr->timestamp,
 					(unsigned long long)lr->owner_id,
@@ -505,19 +740,23 @@ int direct_dump(struct task *task, char *dump_path, int force_mode)
 
 				if (force_mode) {
 					bitmap = (char *)lr_end + LEADER_RECORD_MAX;
-					for (b = 0; b < DEFAULT_MAX_HOSTS; b++) {
+					for (b = 0; b < max_hosts; b++) {
 						if (test_id_bit(b+1, bitmap))
 							printf(" %d", b+1);
 					}
 				}
 				printf("\n");
 			}
-		} else if (lr->magic == PAXOS_DISK_MAGIC) {
+		} else if (magic == PAXOS_DISK_MAGIC) {
+			lr_end = (struct leader_record *)data;
+			leader_record_in(lr_end, &lr_in);
+			lr = &lr_in;
+
 			strncpy(sname, lr->space_name, NAME_ID_SIZE);
 			strncpy(rname, lr->resource_name, NAME_ID_SIZE);
 
 			printf("%08llu %36s %48s %010llu %04llu %04llu %llu",
-			       (unsigned long long)(sector_nr * sd.sector_size),
+			       (unsigned long long)(start_offset + (sector_nr * sector_size)),
 			       sname, rname,
 			       (unsigned long long)lr->timestamp,
 			       (unsigned long long)lr->owner_id,
@@ -525,7 +764,7 @@ int direct_dump(struct task *task, char *dump_path, int force_mode)
 			       (unsigned long long)lr->lver);
 
 			if (force_mode) {
-				struct request_record *rr_end = (struct request_record *)(data + sd.sector_size);
+				struct request_record *rr_end = (struct request_record *)(data + sector_size);
 				request_record_in(rr_end, &rr);
 				printf("/%llu/%u",
 				       (unsigned long long)rr.lver, rr.force_mode);
@@ -533,7 +772,7 @@ int direct_dump(struct task *task, char *dump_path, int force_mode)
 			printf("\n");
 
 			for (i = 0; i < lr->num_hosts; i++) {
-				char *pd_end = data + ((2 + i) * sd.sector_size);
+				char *pd_end = data + ((2 + i) * sector_size);
 				struct mode_block *mb_end = (struct mode_block *)(pd_end + MBLOCK_OFFSET);
 
 				if (force_mode > 1) {
@@ -560,11 +799,49 @@ int direct_dump(struct task *task, char *dump_path, int force_mode)
 				printf("                                                                                                          ");
 				printf("%04u %04llu SH\n", i+1, (unsigned long long)mb.generation);
 			}
+		} else if (magic == RINDEX_DISK_MAGIC) {
+			rh_end = (struct rindex_header *)data;
+			rindex_header_in(rh_end, &rh_in);
+			rh = &rh_in;
+
+			strncpy(sname, rh->lockspace_name, NAME_ID_SIZE);
+
+			printf("%08llu %36s rindex_header 0x%x %d %u %llu\n",
+			       (unsigned long long)(start_offset + (sector_nr * sector_size)),
+			       sname,
+			       rh->flags, rh->sector_size, rh->max_resources,
+			       (unsigned long long)rh->rx_offset);
+
+			if (!force_mode)
+				goto next;
+
+			/* i begins with 1 to skip the first sector of the rindex which holds the header */
+
+			for (i = 1; i < sector_count; i++) {
+				int entry_size = sizeof(struct rindex_entry);
+				int entries_per_sector = sector_size / entry_size;
+
+				for (j = 0; j < entries_per_sector; j++) {
+					re_end = (struct rindex_entry *)(data + (i * sector_size) + (j * entry_size));
+					rindex_entry_in(re_end, &re_in);
+					re = &re_in;
+
+					if (!re->res_offset && !re->name[0])
+						continue;
+
+					printf("%08llu %36s rentry %s %llu\n",
+			       			(unsigned long long)(start_offset + ((sector_nr * sector_size) + (i * sector_size) + (j * entry_size))),
+			       			sname,
+						re->name, (unsigned long long)re->res_offset);
+				}
+			}
+
+
 		} else {
 			if (end_sector_nr == 0)
 				break;
 		}
-
+ next:
 		sector_nr += sector_count;
 	}
 
@@ -583,7 +860,7 @@ int direct_next_free(struct task *task, char *path)
 	struct leader_record lr;
 	struct sync_disk sd;
 	uint64_t sector_nr;
-	int sector_count, datalen, align_size;
+	int sector_size, sector_count, datalen, align_size;
 	int rv;
 
 	memset(&sd, 0, sizeof(struct sync_disk));
@@ -602,13 +879,19 @@ int direct_next_free(struct task *task, char *path)
 	if (rv < 0)
 		return -ENODEV;
 
-	rv = direct_align(&sd);
-	if (rv < 0)
-		goto out_close;
+	if (com.sector_size)
+		sector_size = com.sector_size;
+        if (com.align_size)
+                align_size = com.align_size;
 
-	align_size = rv;
-	datalen = sd.sector_size;
-	sector_count = align_size / sd.sector_size;
+	if (!sector_size || !align_size) {
+		rv = direct_read_leader_sizes(task, &sd, &sector_size, &align_size);
+		if (rv < 0)
+			return rv;
+	}
+
+	sector_count = align_size / sector_size;
+	datalen = sector_size;
 
 	data = malloc(datalen);
 	if (!data) {
@@ -620,17 +903,17 @@ int direct_next_free(struct task *task, char *path)
 	rv = -ENOSPC;
 
 	while (1) {
-		memset(data, 0, sd.sector_size);
+		memset(data, 0, sector_size);
 
-		rv = read_sectors(&sd, sector_nr, 1, data, datalen,
+		rv = read_sectors(&sd, sector_size, sector_nr, 1, data, datalen,
 				  task, DEFAULT_IO_TIMEOUT, "next_free");
 
 		lr_end = (struct leader_record *)data;
 
 		leader_record_in(lr_end, &lr);
 
-		if (lr.magic != DELTA_DISK_MAGIC && lr.magic != PAXOS_DISK_MAGIC) {
-			printf("%llu\n", (unsigned long long)(sector_nr * sd.sector_size));
+		if (lr.magic != DELTA_DISK_MAGIC && lr.magic != PAXOS_DISK_MAGIC && lr.magic != RINDEX_DISK_MAGIC) {
+			printf("%llu\n", (unsigned long long)(sector_nr * sector_size));
 			rv = 0;
 			goto out_free;
 		}
@@ -644,3 +927,43 @@ int direct_next_free(struct task *task, char *path)
 	return rv;
 }
 
+
+int direct_rindex_format(struct task *task, struct sanlk_rindex *ri)
+{
+	return rindex_format(task, ri);
+}
+
+int direct_rindex_rebuild(struct task *task, struct sanlk_rindex *ri,
+			  uint32_t cmd_flags)
+{
+	return rindex_rebuild(task, ri, cmd_flags | SANLK_RX_NO_LOCKSPACE);
+}
+
+int direct_rindex_lookup(struct task *task, struct sanlk_rindex *ri,
+			 struct sanlk_rentry *re, uint32_t cmd_flags)
+{
+	struct sanlk_rentry re_ret;
+	int rv;
+
+	rv = rindex_lookup(task, ri, re, &re_ret, cmd_flags | SANLK_RX_NO_LOCKSPACE);
+
+	if (!rv)
+		memcpy(re, &re_ret, sizeof(re_ret));
+
+	return rv;
+}
+
+int direct_rindex_update(struct task *task, struct sanlk_rindex *ri,
+			 struct sanlk_rentry *re, uint32_t cmd_flags)
+{
+	struct sanlk_rentry re_ret;
+	int rv;
+
+	rv = rindex_update(task, ri, re, &re_ret, cmd_flags | SANLK_RX_NO_LOCKSPACE);
+
+	if (!rv)
+		memcpy(re, &re_ret, sizeof(re_ret));
+
+	return rv;
+}
+
