@@ -44,6 +44,7 @@ static pthread_t resource_pt;
 static int resource_thread_stop;
 static int resource_thread_work;
 static int resource_thread_work_examine;
+static struct list_head resources_free;
 static struct list_head resources_held;
 static struct list_head resources_add;
 static struct list_head resources_rem;
@@ -51,13 +52,78 @@ static struct list_head resources_orphan;
 static pthread_mutex_t resource_mutex;
 static pthread_cond_t resource_cond;
 static struct list_head host_events;
+static int resources_free_count;
+static uint32_t resource_id_counter = 2; /* id 1 used for internal rindex lease */
 
+#define FREE_RES_COUNT 128
+
+/*
+ * There's not much advantage to saving resource structs and reusing them again
+ * when they are requested again.  One advantage can be that the res_id remains
+ * unchanged for frequently requested resources, so a new resource description
+ * isn't logged each time it's requested.  There may be some other
+ * optimizations that could be added.  We may want per-lockspace lists of
+ * resources, or purge free resources when lockspaces are removed.
+ */
 
 static void free_resource(struct resource *r)
 {
+	struct resource *rtmp = NULL;
+	struct resource *rmin = NULL;
+
 	if (r->lvb)
 		free(r->lvb);
-	free(r);
+
+	if (resources_free_count < FREE_RES_COUNT) {
+		resources_free_count++;
+		list_add(&r->list, &resources_free);
+		return;
+	}
+
+	/* the max are being saved, free the least used before saving this one */
+
+	list_for_each_entry_reverse(rtmp, &resources_free, list) {
+		if (!rtmp->reused) {
+			list_del(&rtmp->list);
+			free(rtmp);
+			goto out;
+		}
+
+		if (!rmin || (rtmp->reused < rmin->reused))
+			rmin = rtmp;
+	}
+
+	if (rmin) {
+		list_del(&rmin->list);
+		free(rmin);
+	}
+ out:
+	list_add(&r->list, &resources_free);
+}
+
+static struct resource *get_free_resource(struct token *token, int *token_matches)
+{
+	struct resource *r;
+
+	/* find a previous r that matches token */
+	list_for_each_entry(r, &resources_free, list) {
+		if (strcmp(r->r.lockspace_name, token->r.lockspace_name))
+			continue;
+		if (strcmp(r->r.name, token->r.name))
+			continue;
+		if (r->r.num_disks != token->r.num_disks)
+			continue;
+		if (strcmp(r->r.disks[0].path, token->r.disks[0].path))
+			continue;
+
+		*token_matches = 1;
+		resources_free_count--;
+		list_del(&r->list);
+		r->reused++;
+		return r;
+	}
+
+	return NULL;
 }
 
 /* N.B. the reporting function looks for the
@@ -81,10 +147,10 @@ void send_state_resources(int fd)
 	}
 
 	list_for_each_entry(r, &resources_rem, list)
-		send_state_resource(fd, r, "rem", r->pid, r->release_token_id);
+		send_state_resource(fd, r, "rem", r->pid, 0);
 
 	list_for_each_entry(r, &resources_orphan, list)
-		send_state_resource(fd, r, "orphan", r->pid, r->release_token_id);
+		send_state_resource(fd, r, "orphan", r->pid, 0);
 	pthread_mutex_unlock(&resource_mutex);
 }
 
@@ -103,13 +169,24 @@ int read_resource_owners(struct task *task, struct token *token,
 	char *lease_buf_dblock;
 	char *lease_buf = NULL;
 	char *hosts_buf = NULL;
+	const int sector_size_set = token->sector_size != 0;
+	const int align_size_set = token->align_size != 0;
+	int align_size;
 	int host_count = 0;
 	int i, rv;
 
 	disk = &token->disks[0];
 
-	/* we could in-line paxos_read_buf here like we do in read_mode_block */
+	/* If sector size not set, start with the smaller one. */
+	if (!sector_size_set)
+		token->sector_size = 512;
 
+	/* If align size not set, default to the older align. */
+	if (!align_size_set)
+		token->align_size = sector_size_to_align_size_old(token->sector_size);
+
+	/* we could in-line paxos_read_buf here like we do in read_mode_block */
+ retry:
 	rv = paxos_read_buf(task, token, &lease_buf);
 	if (rv < 0) {
 		log_errot(token, "read_resource_owners read_buf rv %d", rv);
@@ -125,6 +202,43 @@ int read_resource_owners(struct task *task, struct token *token,
 
 	leader_record_in(&leader_end, &leader);
 
+	align_size = leader_align_size_from_flag(leader.flags);
+	if (!align_size)
+		align_size = sector_size_to_align_size_old(leader.sector_size);
+
+	/* If caller specified values are incorrect, fail. */
+
+	if (sector_size_set && token->sector_size != leader.sector_size) {
+		log_errot(token, "read_resource_owners invalid sector_size: %d actual: %d",
+			  token->sector_size, leader.sector_size);
+		rv = -EINVAL;
+		goto out;
+	}
+
+	if (align_size_set && token->align_size != align_size) {
+		log_errot(token, "read_resource_owners invalid align_size: %d actual: %d",
+			  token->align_size, align_size);
+		rv = -EINVAL;
+		goto out;
+	}
+
+	/*
+	 * If the caller did not specify sector size and our guess was wrong,
+	 * retry with the actual value
+	 */
+	if ((token->sector_size != leader.sector_size) ||
+	    (token->align_size != align_size)) {
+		log_debug("read_resource_owners rereading with correct sizes");
+		token->sector_size = leader.sector_size;
+		token->align_size  = align_size;
+		free(lease_buf);
+		lease_buf = NULL;
+		goto retry;
+	}
+
+	token->sector_size = leader.sector_size;
+	token->align_size = align_size;
+
 	rv = paxos_verify_leader(token, disk, &leader, checksum, "read_resource_owners");
 	if (rv < 0)
 		goto out;
@@ -135,7 +249,7 @@ int read_resource_owners(struct task *task, struct token *token,
 		host_count++;
 
 	for (i = 0; i < leader.num_hosts; i++) {
-		lease_buf_dblock = lease_buf + ((2 + i) * disk->sector_size);
+		lease_buf_dblock = lease_buf + ((2 + i) * token->sector_size);
 		mb_end = (struct mode_block *)(lease_buf_dblock + MBLOCK_OFFSET);
 
 		mode_block_in(mb_end, &mb);
@@ -187,7 +301,7 @@ int read_resource_owners(struct task *task, struct token *token,
 	}
 
 	for (i = 0; i < leader.num_hosts; i++) {
-		lease_buf_dblock = lease_buf + ((2 + i) * disk->sector_size);
+		lease_buf_dblock = lease_buf + ((2 + i) * token->sector_size);
 		mb_end = (struct mode_block *)(lease_buf_dblock + MBLOCK_OFFSET);
 
 		mode_block_in(mb_end, &mb);
@@ -218,7 +332,7 @@ int read_resource_owners(struct task *task, struct token *token,
    enough knowledge to say it's safely dead (unless of course we find it is
    alive while waiting) */
 
-static int host_live(char *lockspace_name, uint64_t host_id, uint64_t gen)
+static int host_live(char *lockspace_name, uint32_t space_id, uint64_t host_id, uint64_t gen)
 {
 	struct host_status hs;
 	uint64_t now;
@@ -227,28 +341,28 @@ static int host_live(char *lockspace_name, uint64_t host_id, uint64_t gen)
 
 	rv = host_info(lockspace_name, host_id, &hs);
 	if (rv) {
-		log_debug("host_live %llu %llu yes host_info %d",
-			  (unsigned long long)host_id, (unsigned long long)gen, rv);
+		log_sid(space_id, "host_live %llu %llu yes host_info %d",
+			(unsigned long long)host_id, (unsigned long long)gen, rv);
 		return 1;
 	}
 
 	if (!hs.last_check) {
-		log_debug("host_live %llu %llu yes unchecked",
-			  (unsigned long long)host_id, (unsigned long long)gen);
+		log_sid(space_id, "host_live %llu %llu yes unchecked",
+			(unsigned long long)host_id, (unsigned long long)gen);
 		return 1;
 	}
 
 	/* the host_id lease is free, not being used */
 	if (!hs.timestamp) {
-		log_debug("host_live %llu %llu no lease free",
-			  (unsigned long long)host_id, (unsigned long long)gen);
+		log_sid(space_id, "host_live %llu %llu no lease free",
+			(unsigned long long)host_id, (unsigned long long)gen);
 		return 0;
 	}
 
 	if (hs.owner_generation > gen) {
-		log_debug("host_live %llu %llu no old gen %llu",
-			  (unsigned long long)host_id, (unsigned long long)gen,
-			  (unsigned long long)hs.owner_generation);
+		log_sid(space_id, "host_live %llu %llu no old gen %llu",
+			(unsigned long long)host_id, (unsigned long long)gen,
+			(unsigned long long)hs.owner_generation);
 		return 0;
 	}
 
@@ -258,23 +372,23 @@ static int host_live(char *lockspace_name, uint64_t host_id, uint64_t gen)
 	other_host_dead_seconds = calc_host_dead_seconds(other_io_timeout);
 
 	if (!hs.last_live && (now - hs.first_check > other_host_dead_seconds)) {
-		log_debug("host_live %llu %llu no first_check %llu",
-			  (unsigned long long)host_id, (unsigned long long)gen,
-			  (unsigned long long)hs.first_check);
+		log_sid(space_id, "host_live %llu %llu no first_check %llu",
+			(unsigned long long)host_id, (unsigned long long)gen,
+			(unsigned long long)hs.first_check);
 		return 0;
 	}
 
 	if (hs.last_live && (now - hs.last_live > other_host_dead_seconds)) {
-		log_debug("host_live %llu %llu no last_live %llu",
-			  (unsigned long long)host_id, (unsigned long long)gen,
-			  (unsigned long long)hs.last_live);
+		log_sid(space_id, "host_live %llu %llu no last_live %llu",
+			(unsigned long long)host_id, (unsigned long long)gen,
+			(unsigned long long)hs.last_live);
 		return 0;
 	}
 
-	log_debug("host_live %llu %llu yes recent first_check %llu last_live %llu",
-		  (unsigned long long)host_id, (unsigned long long)gen,
-		  (unsigned long long)hs.first_check,
-		  (unsigned long long)hs.last_live);
+	log_sid(space_id, "host_live %llu %llu yes recent first_check %llu last_live %llu",
+		(unsigned long long)host_id, (unsigned long long)gen,
+		(unsigned long long)hs.first_check,
+		(unsigned long long)hs.last_live);
 
 	return 1;
 }
@@ -313,7 +427,7 @@ static int write_host_block(struct task *task, struct token *token,
 
 	disk = &token->disks[0];
 
-	iobuf_len = disk->sector_size;
+	iobuf_len = token->sector_size;
 	if (!iobuf_len)
 		return -EINVAL;
 
@@ -330,9 +444,25 @@ static int write_host_block(struct task *task, struct token *token,
 	 * values intact because other hosts may be running the
 	 * paxos algorithm and these values need to remain intact
 	 * for them to reach the correct result.
+	 *
+	 * Be very careful that the latest/correct copy of our
+	 * dblock values are being used here.  A paxos ballot
+	 * can get confused/stuck if we write the wrong dblock
+	 * values.
 	 */
 	if (pd) {
-		paxos_dblock_out(pd, &pd_end);
+		if (pd->inp && (pd->inp != token->host_id)) {
+			/* This should never happen, sanity check. */
+			log_errot(token, "Ignore bad dblock while writing mblock %llu:%llu:%llu:%llu",
+				  (unsigned long long)pd->inp,
+				  (unsigned long long)pd->inp2,
+				  (unsigned long long)pd->inp3,
+				  (unsigned long long)pd->lver);
+			memset(pd, 0, sizeof(struct paxos_dblock));
+		} else {
+			paxos_dblock_out(pd, &pd_end);
+		}
+
 		checksum = dblock_checksum(&pd_end);
 		pd->checksum = checksum;
 		pd_end.checksum = cpu_to_le32(checksum);
@@ -350,7 +480,7 @@ static int write_host_block(struct task *task, struct token *token,
 	for (d = 0; d < num_disks; d++) {
 		disk = &token->disks[d];
 
-		offset = disk->offset + ((2 + host_id - 1) * disk->sector_size);
+		offset = disk->offset + ((2 + host_id - 1) * token->sector_size);
 
 		rv = write_iobuf(disk->fd, offset, iobuf, iobuf_len, task, token->io_timeout, NULL);
 		if (rv < 0)
@@ -419,7 +549,7 @@ static int read_mode_block(struct task *task, struct token *token,
 
 	disk = &token->disks[0];
 
-	iobuf_len = disk->sector_size;
+	iobuf_len = token->sector_size;
 	if (!iobuf_len)
 		return -EINVAL;
 
@@ -432,7 +562,7 @@ static int read_mode_block(struct task *task, struct token *token,
 	for (d = 0; d < num_disks; d++) {
 		disk = &token->disks[d];
 
-		offset = disk->offset + ((2 + host_id - 1) * disk->sector_size);
+		offset = disk->offset + ((2 + host_id - 1) * token->sector_size);
 
 		rv = read_iobuf(disk->fd, offset, iobuf, iobuf_len, task, token->io_timeout, NULL);
 		if (rv < 0)
@@ -497,7 +627,7 @@ static int clear_dead_shared(struct task *task, struct token *token,
 			continue;
 		}
 
-		if (host_live(token->r.lockspace_name, host_id, mb.generation)) {
+		if (host_live(token->r.lockspace_name, token->space_id, host_id, mb.generation)) {
 			log_token(token, "clear_dead_shared host_id %llu gen %llu alive",
 				  (unsigned long long)host_id, (unsigned long long)mb.generation);
 			live++;
@@ -537,9 +667,9 @@ static int read_lvb_block(struct task *task, struct token *token)
 
 	r = token->resource;
 	disk = &token->disks[0];
-	iobuf_len = disk->sector_size;
+	iobuf_len = token->sector_size;
 	iobuf = r->lvb;
-	offset = disk->offset + (LVB_SECTOR * disk->sector_size);
+	offset = disk->offset + (LVB_SECTOR * token->sector_size);
 
 	if (!r->lvb)
 		return 0;
@@ -557,9 +687,9 @@ static int write_lvb_block(struct task *task, struct resource *r, struct token *
 	int iobuf_len, rv;
 
 	disk = &token->disks[0];
-	iobuf_len = disk->sector_size;
+	iobuf_len = token->sector_size;
 	iobuf = r->lvb;
-	offset = disk->offset + (LVB_SECTOR * disk->sector_size);
+	offset = disk->offset + (LVB_SECTOR * token->sector_size);
 
 	if (!r->lvb)
 		return 0;
@@ -907,8 +1037,7 @@ static int _release_token(struct task *task, struct token *token,
 			retry_async = 1;
 
 		/* want to see this result in sanlock.log but not worry people with error */
-		log_level(0, token->token_id, NULL, LOG_WARNING,
-			  "release_token erase all leader lver %llu rv %d",
+		log_warnt(token, "release_token erase all leader lver %llu rv %d",
 			  (unsigned long long)lver, rv);
 
 	} else if (r_flags & R_UNDO_SHARED) {
@@ -986,8 +1115,8 @@ static int _release_token(struct task *task, struct token *token,
 			log_token(token, "release_token done r_flags %x", r_flags);
 		pthread_mutex_lock(&resource_mutex);
 		list_del(&r->list);
-		pthread_mutex_unlock(&resource_mutex);
 		free_resource(r);
+		pthread_mutex_unlock(&resource_mutex);
 		return ret;
 	}
 
@@ -1002,7 +1131,6 @@ static int _release_token(struct task *task, struct token *token,
 	log_errot(token, "release_token timeout r_flags %x", r_flags);
 	pthread_mutex_lock(&resource_mutex);
 	r->flags |= R_THREAD_RELEASE;
-	r->release_token_id = token->token_id;
 	pthread_mutex_unlock(&resource_mutex);
 	return SANLK_AIO_TIMEOUT;
 }
@@ -1041,11 +1169,9 @@ void release_token_async(struct token *token)
 			list_del(&r->list);
 			free_resource(r);
 		} else if (token->acquire_flags & SANLK_RES_PERSISTENT) {
-			r->release_token_id = token->token_id;
 			list_move(&r->list, &resources_orphan);
 		} else {
 			r->flags |= R_THREAD_RELEASE;
-			r->release_token_id = token->token_id;
 			resource_thread_work = 1;
 			list_move(&r->list, &resources_rem);
 			pthread_cond_signal(&resource_cond);
@@ -1137,21 +1263,41 @@ static void copy_disks(void *dst, void *src, int num_disks)
 	}
 }
 
-static struct resource *new_resource(struct token *token)
+static struct resource *get_resource(struct token *token, int *new_id)
 {
 	struct resource *r;
+	int token_matches = 0;
+	uint32_t res_id = 0;
+	uint32_t reused = 0;
 	int disks_len, r_len;
 
 	disks_len = token->r.num_disks * sizeof(struct sync_disk);
 	r_len = sizeof(struct resource) + disks_len;
 
-	r = malloc(r_len);
-	if (!r)
-		return NULL;
+	r = get_free_resource(token, &token_matches);
+
+	if (r && token_matches) {
+		res_id = r->res_id;
+		reused = r->reused;
+		*new_id = 0;
+	} else if (r) {
+		res_id = resource_id_counter++;
+		*new_id = 1;
+	} else {
+		r = malloc(r_len);
+		if (!r)
+			return NULL;
+		res_id = resource_id_counter++;
+		*new_id = 1;
+	}
 
 	memset(r, 0, r_len);
-	memcpy(&r->r, &token->r, sizeof(struct sanlk_resource));
 
+	/* preserved from one use to the next */
+	r->res_id = res_id;
+	r->reused = reused;
+
+	memcpy(&r->r, &token->r, sizeof(struct sanlk_resource));
 	r->io_timeout = token->io_timeout;
 
 	/* disks copied after open_disks because open_disks sets sector_size
@@ -1487,8 +1633,11 @@ int acquire_token(struct task *task, struct token *token, uint32_t cmd_flags,
 	int allow_orphan = 0;
 	int only_orphan = 0;
 	int owner_nowait = 0;
+	int new_id = 0;
 	int rv;
 
+	memset(&dblock, 0, sizeof(dblock));
+
 	if (token->acquire_flags & SANLK_RES_LVER)
 		acquire_lver = token->acquire_lver;
 	if (token->acquire_flags & SANLK_RES_NUM_HOSTS)
@@ -1509,6 +1658,7 @@ int acquire_token(struct task *task, struct token *token, uint32_t cmd_flags,
 
 	r = find_resource(token, &resources_rem);
 	if (r) {
+		token->res_id = r->res_id;
 		if (!com.quiet_fail)
 			log_errot(token, "acquire_token resource being removed");
 		pthread_mutex_unlock(&resource_mutex);
@@ -1517,6 +1667,7 @@ int acquire_token(struct task *task, struct token *token, uint32_t cmd_flags,
 
 	r = find_resource(token, &resources_add);
 	if (r) {
+		token->res_id = r->res_id;
 		if (!com.quiet_fail)
 			log_errot(token, "acquire_token resource being added");
 		pthread_mutex_unlock(&resource_mutex);
@@ -1526,6 +1677,7 @@ int acquire_token(struct task *task, struct token *token, uint32_t cmd_flags,
 	r = find_resource(token, &resources_held);
 	if (r && (token->acquire_flags & SANLK_RES_SHARED) && (r->flags & R_SHARED)) {
 		/* multiple shared holders allowed */
+		token->res_id = r->res_id;
 		log_token(token, "acquire_token add shared");
 		copy_disks(&token->r.disks, &r->r.disks, token->r.num_disks);
 		token->resource = r;
@@ -1535,6 +1687,7 @@ int acquire_token(struct task *task, struct token *token, uint32_t cmd_flags,
 	}
 
 	if (r) {
+		token->res_id = r->res_id;
 		if (!com.quiet_fail)
 			log_errot(token, "acquire_token resource exists");
 		pthread_mutex_unlock(&resource_mutex);
@@ -1545,6 +1698,7 @@ int acquire_token(struct task *task, struct token *token, uint32_t cmd_flags,
 
 	r = find_resource(token, &resources_orphan);
 	if (r && !allow_orphan) {
+		token->res_id = r->res_id;
 		log_errot(token, "acquire_token found orphan");
 		pthread_mutex_unlock(&resource_mutex);
 		return -EUCLEAN;
@@ -1554,6 +1708,7 @@ int acquire_token(struct task *task, struct token *token, uint32_t cmd_flags,
 
 	if (r && allow_orphan && 
 	    (r->flags & R_SHARED) && !(token->acquire_flags & SANLK_RES_SHARED)) {
+		token->res_id = r->res_id;
 		log_errot(token, "acquire_token orphan is shared");
 		pthread_mutex_unlock(&resource_mutex);
 		return -EUCLEAN;
@@ -1563,6 +1718,7 @@ int acquire_token(struct task *task, struct token *token, uint32_t cmd_flags,
 
 	if (r && allow_orphan &&
 	    !(r->flags & R_SHARED) && (token->acquire_flags & SANLK_RES_SHARED)) {
+		token->res_id = r->res_id;
 		log_errot(token, "acquire_token orphan is exclusive");
 		pthread_mutex_unlock(&resource_mutex);
 		return -EUCLEAN;
@@ -1572,6 +1728,7 @@ int acquire_token(struct task *task, struct token *token, uint32_t cmd_flags,
 
 	if (r && allow_orphan && 
 	    (r->flags & R_SHARED) && (token->acquire_flags & SANLK_RES_SHARED)) {
+		token->res_id = r->res_id;
 		log_token(token, "acquire_token adopt shared orphan");
 		token->resource = r;
 		list_add(&token->list, &r->tokens);
@@ -1594,6 +1751,7 @@ int acquire_token(struct task *task, struct token *token, uint32_t cmd_flags,
 
 	if (r && allow_orphan &&
 	    !(r->flags & R_SHARED) && !(token->acquire_flags & SANLK_RES_SHARED)) {
+		token->res_id = r->res_id;
 		log_token(token, "acquire_token adopt orphan");
 		token->r.lver = r->leader.lver;
 		r->pid = token->pid;
@@ -1625,7 +1783,7 @@ int acquire_token(struct task *task, struct token *token, uint32_t cmd_flags,
 	 * The resource does not exist, so create it.
 	 */
 
-	r = new_resource(token);
+	r = get_resource(token, &new_id);
 	if (!r) {
 		pthread_mutex_unlock(&resource_mutex);
 		return -ENOMEM;
@@ -1635,9 +1793,19 @@ int acquire_token(struct task *task, struct token *token, uint32_t cmd_flags,
 	memcpy(r->killargs, killargs, SANLK_HELPER_ARGS_LEN);
 	list_add(&token->list, &r->tokens);
 	list_add(&r->list, &resources_add);
+	token->res_id = r->res_id;
 	token->resource = r;
 	pthread_mutex_unlock(&resource_mutex);
 
+	if (new_id) {
+		/* save a record of what this id is for later debugging */
+		log_warnt(token, "resource %.48s:%.48s:%.256s:%llu",
+			  token->r.lockspace_name,
+			  token->r.name,
+			  token->r.disks[0].path,
+			  (unsigned long long)token->r.disks[0].offset);
+	}
+
 	rv = open_disks(token->disks, token->r.num_disks);
 	if (rv < 0) {
 		log_errot(token, "acquire_token open error %d", rv);
@@ -1647,23 +1815,19 @@ int acquire_token(struct task *task, struct token *token, uint32_t cmd_flags,
 
 	copy_disks(&r->r.disks, &token->r.disks, token->r.num_disks);
 
-	if (cmd_flags & SANLK_ACQUIRE_LVB) {
-		char *iobuf, **p_iobuf;
-		p_iobuf = &iobuf;
-
-		rv = posix_memalign((void *)p_iobuf, getpagesize(), token->disks[0].sector_size);
-		if (rv)
-			log_errot(token, "acquire_token lvb size %d memalign error %d",
-				  token->disks[0].sector_size, rv);
-		else
-			r->lvb = iobuf;
-	}
-
  retry:
 	memset(&leader, 0, sizeof(struct leader_record));
 
 	rv = acquire_disk(task, token, acquire_lver, new_num_hosts, owner_nowait, &leader, &dblock);
 
+	/*
+	 * token sector_size/align_size starts by using sector_size/align_size
+	 * from the ls, but can change in paxos acquire when we see what's in
+	 * the leader_record.
+	 */
+	r->sector_size = token->sector_size;
+	r->align_size = token->align_size;
+
 	if (rv == SANLK_ACQUIRE_IDLIVE || rv == SANLK_ACQUIRE_OWNED || rv == SANLK_ACQUIRE_OTHER) {
 		/*
 		 * Another host owns the lease.  They may be holding for
@@ -1796,11 +1960,22 @@ int acquire_token(struct task *task, struct token *token, uint32_t cmd_flags,
 
  out:
 	if (cmd_flags & SANLK_ACQUIRE_LVB) {
-		rv = read_lvb_block(task, token);
-		if (rv < 0) {
-			/* TODO: we should probably notify the caller somehow about
-			   lvb read/write independent of the lease results. */
-			log_errot(token, "acquire_token read_lvb error %d", rv);
+		char *iobuf, **p_iobuf;
+		p_iobuf = &iobuf;
+
+		/* TODO: we should probably notify the caller somehow about
+		   lvb read/write independent of the lease results. */
+
+		rv = posix_memalign((void *)p_iobuf, getpagesize(), token->sector_size);
+		if (rv) {
+			log_errot(token, "acquire_token lvb size %d memalign error %d",
+				  token->sector_size, rv);
+		} else {
+			r->lvb = iobuf;
+
+			rv = read_lvb_block(task, token);
+			if (rv < 0)
+				log_errot(token, "acquire_token read_lvb error %d", rv);
 		}
 	}
 
@@ -1818,6 +1993,7 @@ int request_token(struct task *task, struct token *token, uint32_t force_mode,
 {
 	struct leader_record leader;
 	struct request_record req;
+	int align_size;
 	int rv;
 
 	memset(&req, 0, sizeof(req));
@@ -1831,10 +2007,29 @@ int request_token(struct task *task, struct token *token, uint32_t force_mode,
 	if (!token->acquire_lver && !force_mode)
 		goto req_read;
 
+	/*
+	 * cmd_request() takes the sector_size and align_size from the
+	 * lockspace as a starting point, setting them in the token.  If the
+	 * leader_record for the paxos lease on disk is different, then adjust
+	 * the values in the token.
+	 */
+
 	rv = paxos_lease_leader_read(task, token, &leader, "request");
 	if (rv < 0)
 		goto out;
 
+	align_size = leader_align_size_from_flag(leader.flags);
+	if (!align_size)
+		align_size = sector_size_to_align_size_old(leader.sector_size);
+
+	if ((leader.sector_size != token->sector_size) || (align_size != token->align_size)) {
+		/* paxos lease has different size than we borrowed from the lockspace */
+		token->sector_size = leader.sector_size;
+		token->align_size = align_size;
+		if (!token->align_size)
+			token->align_size = sector_size_to_align_size_old(leader.sector_size);
+	}
+
 	if (leader.timestamp == LEASE_FREE) {
 		*owner_id = 0;
 		rv = SANLK_OK;
@@ -1886,7 +2081,7 @@ int request_token(struct task *task, struct token *token, uint32_t force_mode,
  out:
 	close_disks(token->disks, token->r.num_disks);
 
-	log_debug("request_token rv %d owner %llu lver %llu mode %u",
+	log_token(token, "request_token rv %d owner %llu lver %llu mode %u",
 		  rv, (unsigned long long)*owner_id,
 		  (unsigned long long)req.lver, req.force_mode);
 
@@ -1917,7 +2112,7 @@ static int examine_token(struct task *task, struct token *token,
 
 	memcpy(req_out, &req, sizeof(struct request_record));
  out:
-	log_debug("examine_token rv %d lver %llu mode %u",
+	log_token(token, "examine_token rv %d lver %llu mode %u",
 		  rv, (unsigned long long)req.lver, req.force_mode);
 
 	return rv;
@@ -1948,7 +2143,7 @@ static void do_request(struct token *tt, int pid, uint32_t force_mode)
 		return;
 	}
 
-	log_debug("do_request %d flags %x %.48s:%.48s",
+	log_token(tt, "do_request %d flags %x %.48s:%.48s",
 		  pid, flags, tt->r.lockspace_name, tt->r.name);
 
 	if (helper_kill_fd == -1) {
@@ -2108,8 +2303,7 @@ static void resource_thread_release(struct task *task, struct resource *r, struc
 			retry_async = 1;
 
 		/* want to see this result in sanlock.log but not worry people with error */
-		log_level(0, token->token_id, NULL, LOG_WARNING,
-			  "release async erase all leader lver %llu rv %d",
+		log_warnt(token, "release async erase all leader lver %llu rv %d",
 			  (unsigned long long)r->leader.lver, rv);
 
 	} else if (r_flags & R_UNDO_SHARED) {
@@ -2168,8 +2362,8 @@ static void resource_thread_release(struct task *task, struct resource *r, struc
 		log_token(token, "release async done r_flags %x", r_flags);
 		pthread_mutex_lock(&resource_mutex);
 		list_del(&r->list);
-		pthread_mutex_unlock(&resource_mutex);
 		free_resource(r);
+		pthread_mutex_unlock(&resource_mutex);
 		return;
 	}
 
@@ -2202,7 +2396,7 @@ static void resource_thread_examine(struct task *task, struct token *tt, int pid
 		return;
 
 	if (req.lver <= lver) {
-		log_debug("examine req lver %llu our lver %llu",
+		log_token(tt, "examine req lver %llu our lver %llu",
 			  (unsigned long long)req.lver, (unsigned long long)lver);
 		return;
 	}
@@ -2309,8 +2503,10 @@ static void *resource_thread(void *arg GNUC_UNUSED)
 			copy_disks(&tt->r.disks, &r->r.disks, r->r.num_disks);
 			tt->host_id = r->host_id;
 			tt->host_generation = r->host_generation;
-			tt->token_id = r->release_token_id;
+			tt->res_id = r->res_id;
 			tt->io_timeout = r->io_timeout;
+			tt->sector_size = r->sector_size;
+			tt->align_size = r->align_size;
 			tt->resource = r;
 
 			/*
@@ -2345,7 +2541,10 @@ static void *resource_thread(void *arg GNUC_UNUSED)
 			copy_disks(&tt->r.disks, &r->r.disks, r->r.num_disks);
 			tt->host_id = r->host_id;
 			tt->host_generation = r->host_generation;
+			tt->res_id = r->res_id;
 			tt->io_timeout = r->io_timeout;
+			tt->sector_size = r->sector_size;
+			tt->align_size = r->align_size;
 			pid = r->pid;
 			lver = r->leader.lver;
 
@@ -2395,21 +2594,32 @@ int release_orphan(struct sanlk_resource *res)
 	return count;
 }
 
-void purge_resource_orphans(char *space_name)
+static void purge_resource_list(struct list_head *head, char *space_name, const char *list_name)
 {
 	struct resource *r, *safe;
 
 	pthread_mutex_lock(&resource_mutex);
-	list_for_each_entry_safe(r, safe, &resources_orphan, list) {
+	list_for_each_entry_safe(r, safe, head, list) {
 		if (strncmp(r->r.lockspace_name, space_name, NAME_ID_SIZE))
 			continue;
-		log_debug("purge orphan %.48s:%.48s", r->r.lockspace_name, r->r.name);
+		if (list_name)
+			log_debug("purge %s %.48s:%.48s", list_name, r->r.lockspace_name, r->r.name);
 		list_del(&r->list);
-		free_resource(r);
+		free(r);
 	}
 	pthread_mutex_unlock(&resource_mutex);
 }
 
+void purge_resource_orphans(char *space_name)
+{
+	purge_resource_list(&resources_orphan, space_name, "orphan_list");
+}
+
+void purge_resource_free(char *space_name)
+{
+	purge_resource_list(&resources_free, space_name, "free_list");
+}
+
 /*
  * This is called by the main_loop once a second during normal operation.
  * The resources_rem list should normally be empty, so this does nothing.
@@ -2417,7 +2627,7 @@ void purge_resource_orphans(char *space_name)
  * that had timed out previously and need to be retried.
  */
 
-void free_resources(void)
+void rem_resources(void)
 {
 	pthread_mutex_lock(&resource_mutex);
 	if (!list_empty(&resources_rem) && !resource_thread_work) {
@@ -2436,6 +2646,7 @@ int setup_token_manager(void)
 	INIT_LIST_HEAD(&resources_add);
 	INIT_LIST_HEAD(&resources_rem);
 	INIT_LIST_HEAD(&resources_held);
+	INIT_LIST_HEAD(&resources_free);
 	INIT_LIST_HEAD(&resources_orphan);
 	INIT_LIST_HEAD(&host_events);
 
